// Example 9-5. Simple use of function pointers
// functionPointers.c

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

static double Add(double x, double y)
{
    return x + y;
}
static double Sub(double x, double y)
{
    return x - y;
}
static double Mul(double x, double y)
{
    return x * y;
}
static double Div(double x, double y)
{
    return x / y;
}

// Array of 5 pointers to functions that take two double parameters
// and return a double:

// double (*funcTable[5])(double, double)
//          = {Add, Sub, Mul, Div, pow};       // Initializer list.

typedef double func_t(double, double);    // The functions' type is
// now named func_t.
#define FUNC_NUM (5)
func_t* const funcTable[FUNC_NUM] = {Add, Sub, Mul, Div, pow};

// An array of pointers to strings for output:
const char* msgTable[FUNC_NUM] = {"Sum", "Difference", "Product", "Quotient", "Power"};

int main(void)
{
    printf("Enter two operands for some arithmetic:\n");
    double x = 0.0;
    double y = 0.0;
    if (scanf("%lf %lf", &x, &y) != 2)
    {
        printf("Invalid input.\n");
    }
    for (int i = 0; i < FUNC_NUM; ++i)
    {
        printf("%10s: %6.2f\n", msgTable[i], funcTable[i](x, y));
    }

    return 0;
}
