// Example 12-1. The getLine() function
// getLine.c

#include <stdio.h>
#include <stdlib.h>

// Read a line of text from stdin into a dynamically allocated buffer.
// Replace the newline character with a string terminator.
// Arguments:    The maximum line length to read.
// Return value: A pointer to the string read, or
//               NULL if end-of-file was read or if an error occurred.

char* getLine(const unsigned int len_max)
{
    char* linePtr = (char*) malloc(len_max + 1); // Reserve storage for "worst case."
    if (linePtr != NULL)
    {
        // Read a line of text and replace the newline characters with
        // a string terminator:
        int c = EOF;
        unsigned int i = 0U;
        while (i < len_max && (c = getchar()) != '\n' && c != EOF)
        {
            linePtr[i++] = (char)c;
        }
        linePtr[i] = '\0';

        if (c == EOF && i == 0U)         // If end-of-file before any
        {
            // characters were read,
            free(linePtr);                // release the whole buffer.
            linePtr = NULL;
        }
        else                               // Otherwise, release the unused portion.
        {
            linePtr = realloc(linePtr, i + 1U);         // i is the string length.
        }
    }
    return linePtr;
}

int main(void)
{
    char* line = NULL;
    while ((line = getLine(128)) != NULL) // If we can read a line,
    {
        puts(line);                         // process the line,
        free(line);                         // then release the buffer.
    }
    return 0;
}
