// setjmp_ex.c : setjmp() - longjmp() example
// -------------------------------------------------------------

#include <setjmp.h>       // int setjmp(jmp_buf env);
// void longjmp(jmp_buf environment,
//               int returnval);
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

double calculate1(double x);     // Functions defined
double calculate2(double x);     // in calculate.c.

jmp_buf jmp_dest;                // Destination for longjmp()

int main(void)
{
    puts("--- Demonstrating non-local jumps ---\n");

    switch (setjmp(jmp_dest))  // Jump to here for error handling
    {
    case 0:                    // The original setjmp() call
        break;
    case EDOM:                 // Arrived via longjmp() call with EDOM
        puts("Domain error. "
             "Negative numbers are not permitted.");
        break;
    case ERANGE:               // Arrived via longjmp() call with ERANGE
        puts("Range error. "
             "The number you entered is too big.");
        break;
    default:                   // We should never arrive here.
        puts("Unknown error.");
        exit(EXIT_FAILURE);
    }

    printf("Enter a number: ");
    double x = 0;
    int n = 0;
    do
    {
        if ((n = scanf("%lf", &x)) < 0)  // Read in a number.
        {
            exit(EXIT_FAILURE);          // Read end of file.
        }
        while (getchar() != '\n')        // Clear the input buffer.
        {
            ;
        }
        if (n == 0)
        {
            printf("Invalid entry. Try again: ");
        }
    }
    while (n == 0);

    const double y1 = calculate1(x);
    const double y2 = calculate2(x);
    printf("\nResult of Calculation 1: %G\n", y1);
    printf("Result of Calculation 2: %G\n", y2);

    return 0;
}

// calculate.c: Perform some calculations.
// Functions: calculate1(), calculate2().
#include <math.h>
#include <setjmp.h>
#include <errno.h>
extern jmp_buf jmp_dest; // Destination for longjmp()
double calculate1(double x)
{
    if (x < 0.0L)
    {
        longjmp(jmp_dest, EDOM); // Domain error
    }
    else
    {
        return sqrt(x);
    }
}
double calculate2(double x)
{
    const double y = exp(x);
    if (y == HUGE_VAL)
    {
        longjmp(jmp_dest, ERANGE); // Range error
    }
    else
    {
        return y;
    }
}
